searchData={"items":[{"type":"module","title":"Storex","doc":"","ref":"Storex.html"},{"type":"function","title":"Storex.mutate/3","doc":"Mutate store from elixir.\n\nInvoke mutation callback globally across specified store asynchronously:\n```elixir\nStorex.mutate(\"ExampleApp.Store\", \"reload\", [\"params\"])\n```","ref":"Storex.html#mutate/3"},{"type":"function","title":"Storex.mutate/4","doc":"Mutate store from elixir.\n\nInvoke mutation callback by specified key and store asynchronously:\n```elixir\nStorex.mutate(\"user_id\", \"ExampleApp.Store\", \"reload\", [\"params\"])\n```","ref":"Storex.html#mutate/4"},{"type":"module","title":"Storex.Diff","doc":"","ref":"Storex.Diff.html"},{"type":"function","title":"Storex.Diff.check/2","doc":"Check difference between two arguments.\n\n```elixir\nStorex.Diff.check(%{name: \"John\"}, %{name: \"Adam\"})\n[%{a: \"u\", p: [:name], t: \"Adam\"}]\n```\n\nResult explanation:\n```\na: action\n  n - none\n  u - update\n  d - delete\n  i - insert\nt: to\np: path\n```","ref":"Storex.Diff.html#check/2"},{"type":"module","title":"Storex.Plug","doc":"Add Storex to your Plug application, to handle WebSocket connections.\n\nExample for Phoenix Endpoint:\n\n```elixir\ndefmodule YourAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :your_app\n\n  plug Storex.Plug\n\n  # ...\nend\n```","ref":"Storex.Plug.html"},{"type":"module","title":"Options - Storex.Plug","doc":"- `:path` - The path to mount the Storex handler. Default is `\"/storex\"`.","ref":"Storex.Plug.html#module-options"},{"type":"module","title":"Storex.Socket","doc":"Socket connection handler.\n\n  Error codes:\n  - `4000`: Store is not set.\n  - `4001`: Store is not defined or can't be compiled.","ref":"Storex.Socket.html"},{"type":"behaviour","title":"Storex.Store","doc":"","ref":"Storex.Store.html"},{"type":"callback","title":"Storex.Store.init/2","doc":"Called when store session starts.","ref":"Storex.Store.html#c:init/2"},{"type":"callback","title":"Storex.Store.mutation/5","doc":"","ref":"Storex.Store.html#c:mutation/5"},{"type":"callback","title":"Storex.Store.terminate/3","doc":"Called when store session ends.","ref":"Storex.Store.html#c:terminate/3"},{"type":"extras","title":"Storex","doc":"# Storex\n\n![Elixir CI](https://github.com/nerdslabs/storex/workflows/Elixir%20CI/badge.svg) [![Downloads](https://img.shields.io/hexpm/dt/storex.svg)](https://hex.pm/packages/storex)\n\nFrontend store with the state on the backend. You are able to mutate store state from the frontend and also from the backend. Whole communication going through WebSocket.\n\n**Important:** Storex is under active development. Report issues and send proposals [here](https://github.com/nerdslabs/storex/issues/new).\n\nOnly diff of the store state is being sent on each mutation.","ref":"readme.html"},{"type":"extras","title":"Basic usage - Storex","doc":"#","ref":"readme.html#basic-usage"},{"type":"extras","title":"Installation - Storex","doc":"Add **storex** to deps in `mix.exs`:\n\n```elixir\ndefp deps do\n  [{:storex, \"~> 0.4.0\"}]\nend\n```\n\nAlso you need to add **storex** to `package.json` dependencies:\n\n```javascript\n{\n  \"storex\": \"file:../deps/storex\",\n}\n```\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Add storex websocket handler - Storex","doc":"You need to add handler `Storex.Handler.Plug` or `Storex.Handler.Cowboy`.\n\n**Phoenix:**\n```elixir\ndefmodule YourAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :your_app\n\n  plug Storex.Plug, path: \"/storex\"\n\n  # ...\nend\n```\n\n**Cowboy:**\n```elixir\n:cowboy_router.compile([\n  {:_, [\n    # ...\n    {\"/storex\", Storex.Handler.Cowboy, []},\n    # ...\n  ]}\n])\n```\n\n#","ref":"readme.html#add-storex-websocket-handler"},{"type":"extras","title":"Create store - Storex","doc":"To create a store you need to create new elixir module with `init/2` which is called when a page is loaded, every time websocket is connected it generates session_id and passes it as the first argument, params are from Javascript store declaration. `init/2` callback need to return one of this tuples:\n\n- `{:ok, state}` - for initial state\n- `{:ok, state, key}` - for initial state with `key` which can be used as selector for future mutations\n- `{:error, reason}` - to send error message to frontend on initialization\n\nNext, you can declare `mutation/5` where the first argument is mutation name, second is data passed to mutation, next two params are same like in `init/2`, the last one is the current state of the store.\n\n```elixir\ndefmodule ExampleApp.Store.Counter do\n  use Storex.Store\n\n  def init(session_id, params) do\n    {:ok, 0}\n  end\n\n  # `increase` is mutation name, `data` is payload from front-end, `session_id` is current session id of connecton, `initial_params` with which store was initialized, `state` is store current state.\n  def mutation(\"increase\", _data, _session_id, _initial_params, state) do\n    state = state + 1\n\n    {:noreply, state}\n  end\n\n  def mutation(\"decrease\", _data, _session_id, _initial_params, state) do\n    state = state - 1\n\n    {:reply, \"message\", state}\n  end\n\n  def mutation(\"set\", [number], _session_id, _initial_params, state) do\n    {:noreply, number}\n  end\nend\n```\n\n#","ref":"readme.html#create-store"},{"type":"extras","title":"Connect to store - Storex","doc":"You have to connect the newly created store with a frontend side to be able to synchronise the state: `params` are passed as second argument in store `init/2` and as third in `mutation/5`. You can subscribe to changes inside store state by passing option `subscribe` with function as a value.\n\n```javascript\nimport Storex from 'storex'\n\nconst store = new Storex({\n  store: 'ExampleApp.Store.Counter',\n  params: {},\n  subscribe: (state) => {\n    console.log(state)\n  },\n  onConnected() {\n    console.log('connected')\n  },\n  onError(error) {\n    console.log('error', error)\n  },\n  onDisconnected(closeEvent) {\n    console.log('disconnected', closeEvent)\n  }\n})\n```\n\n#","ref":"readme.html#connect-to-store"},{"type":"extras","title":"Mutate store - Storex","doc":"You can mutate store from javascript with store instance:\n\n```javascript\nstore.commit(\"increase\")\nstore.commit(\"decrease\").then((response) => {\n  response // Reply from elixir\n})\nstore.commit(\"set\", 10)\n```\n\nOr directly from elixir:\n\n```elixir\nStorex.mutate(store, \"increase\", [])\nStorex.mutate(store, \"set\", [10])\nStorex.mutate(key, store, \"increase\", [])\nStorex.mutate(key, store, \"set\", [10])\n```\n\n#","ref":"readme.html#mutate-store"},{"type":"extras","title":"Subscribe to store changes - Storex","doc":"You can subscribe to store state changes in javascript with function subscribe:\n\n```javascript\nstore.subscribe((state) => {\n  const state = state\n})\n```\n\nYou can also subscribe to events after store is created:\n\n```javascript\nstore.onConnected(() => {\n  console.log('connected')\n})\n\nstore.onError((error) => {\n  console.log('error', error)\n})\n\nstore.onDisconnected((closeEvent) => {\n  console.log('disconnected', closeEvent)\n})\n```","ref":"readme.html#subscribe-to-store-changes"},{"type":"extras","title":"Configuration - Storex","doc":"#","ref":"readme.html#configuration"},{"type":"extras","title":"Session id generation library - Storex","doc":"You can change library which generate session id for stores. Module needs to have **generate/0** method.\n\n```elixir\nconfig :storex, :session_id_library, Ecto.UUID\n```\n\n#","ref":"readme.html#session-id-generation-library"},{"type":"extras","title":"Default params - Storex","doc":"You can set default params for all stores in Javascript which will be passed to store.\n\n```javascript\nStorex.defaults.params = {\n  jwt: 'someJWT'\n}\n```\n\n#","ref":"readme.html#default-params"},{"type":"extras","title":"Custom store address - Storex","doc":"```javascript\nStorex.defaults.address = 'localhost/stores'\n```","ref":"readme.html#custom-store-address"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}